@@ -26,7 +26,6 @@ import (
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/resource"
 	"k8s.io/apimachinery/pkg/util/intstr"
-	"k8s.io/apimachinery/pkg/util/sets"
 	"knative.dev/pkg/apis"
 	"knative.dev/pkg/ptr"
 	"knative.dev/serving/pkg/apis/config"
@@ -55,6 +54,13 @@ func withPodSpecAffinityEnabled() configOption {
 	}
 }
 
+func withPodSpecHostAliasesEnabled() configOption {
+	return func(cfg *config.Config) *config.Config {
+		cfg.Features.PodSpecHostAliases = config.Enabled
+		return cfg
+	}
+}
+
 func withPodSpecNodeSelectorEnabled() configOption {
 	return func(cfg *config.Config) *config.Config {
 		cfg.Features.PodSpecNodeSelector = config.Enabled
@@ -83,6 +89,27 @@ func withPodSpecSecurityContextEnabled() configOption {
 	}
 }
 
+func withContainerSpecAddCapabilitiesEnabled() configOption {
+	return func(cfg *config.Config) *config.Config {
+		cfg.Features.ContainerSpecAddCapabilities = config.Enabled
+		return cfg
+	}
+}
+
+func withPodSpecVolumesEmptyDirEnabled() configOption {
+	return func(cfg *config.Config) *config.Config {
+		cfg.Features.PodSpecVolumesEmptyDir = config.Enabled
+		return cfg
+	}
+}
+
+func withPodSpecPriorityClassNameEnabled() configOption {
+	return func(cfg *config.Config) *config.Config {
+		cfg.Features.PodSpecPriorityClassName = config.Enabled
+		return cfg
+	}
+}
+
 func TestPodSpecValidation(t *testing.T) {
 	tests := []struct {
 		name    string
@@ -118,6 +145,24 @@ func TestPodSpecValidation(t *testing.T) {
 			}},
 		},
 		want: nil,
+	}, {
+		name: "with automountServiceAccountToken (ok)",
+		ps: corev1.PodSpec{
+			AutomountServiceAccountToken: ptr.Bool(false),
+			Containers: []corev1.Container{{
+				Image: "helloworld",
+			}},
+		},
+		want: nil,
+	}, {
+		name: "with automountServiceAccountToken = true",
+		ps: corev1.PodSpec{
+			AutomountServiceAccountToken: ptr.Bool(true),
+			Containers: []corev1.Container{{
+				Image: "helloworld",
+			}},
+		},
+		want: apis.ErrDisallowedFields("automountServiceAccountToken"),
 	}, {
 		name: "with volume name collision",
 		ps: corev1.PodSpec{
@@ -454,7 +499,7 @@ func TestPodSpecMultiContainerValidation(t *testing.T) {
 		},
 		cfgOpts: []configOption{withPodSpecFieldRefEnabled()},
 		want: &apis.FieldError{
-			Message: "volume with name \"the-name\" not mounted",
+			Message: `volume with name "the-name" not mounted`,
 			Paths:   []string{"volumes[0].name"}},
 	}, {
 		name: "Volume mounts ok when having multiple containers",
@@ -526,7 +571,7 @@ func TestPodSpecMultiContainerValidation(t *testing.T) {
 			},
 		},
 		want: &apis.FieldError{
-			Message: "volume with name \"the-name2\" not mounted",
+			Message: `volume with name "the-name2" not mounted`,
 			Paths:   []string{"volumes[1].name"},
 		}},
 	}
@@ -579,6 +624,19 @@ func TestPodSpecFeatureValidation(t *testing.T) {
 			Paths:   []string{"affinity"},
 		},
 		cfgOpts: []configOption{withPodSpecAffinityEnabled()},
+	}, {
+		name: "HostAliases",
+		featureSpec: corev1.PodSpec{
+			HostAliases: []corev1.HostAlias{{
+				IP:        "127.0.0.1",
+				Hostnames: []string{"foo.remote", "bar.remote"},
+			}},
+		},
+		err: &apis.FieldError{
+			Message: "must not set the field(s)",
+			Paths:   []string{"hostAliases"},
+		},
+		cfgOpts: []configOption{withPodSpecHostAliasesEnabled()},
 	}, {
 		name: "NodeSelector",
 		featureSpec: corev1.PodSpec{
@@ -626,6 +684,16 @@ func TestPodSpecFeatureValidation(t *testing.T) {
 			Paths:   []string{"securityContext"},
 		},
 		cfgOpts: []configOption{withPodSpecSecurityContextEnabled()},
+	}, {
+		name: "PriorityClassName",
+		featureSpec: corev1.PodSpec{
+			PriorityClassName: "foo",
+		},
+		err: &apis.FieldError{
+			Message: "must not set the field(s)",
+			Paths:   []string{"priorityClassName"},
+		},
+		cfgOpts: []configOption{withPodSpecPriorityClassNameEnabled()},
 	}}
 
 	featureTests := []struct {
@@ -810,7 +878,7 @@ func TestContainerValidation(t *testing.T) {
 		name    string
 		c       corev1.Container
 		want    *apis.FieldError
-		volumes sets.String
+		volumes map[string]corev1.Volume
 		cfgOpts []configOption
 	}{{
 		name: "empty container",
@@ -1046,7 +1114,18 @@ func TestContainerValidation(t *testing.T) {
 				ReadOnly:  true,
 			}},
 		},
-		volumes: sets.NewString("the-name"),
+		volumes: map[string]corev1.Volume{
+			"the-name": {
+				Name: "the-name",
+				VolumeSource: corev1.VolumeSource{
+					ConfigMap: &corev1.ConfigMapVolumeSource{
+						LocalObjectReference: corev1.LocalObjectReference{
+							Name: "test-cm",
+						},
+					},
+				},
+			},
+		},
 	}, {
 		name: "has known volumeMounts, but at reserved path",
 		c: corev1.Container{
@@ -1057,7 +1136,18 @@ func TestContainerValidation(t *testing.T) {
 				ReadOnly:  true,
 			}},
 		},
-		volumes: sets.NewString("the-name"),
+		volumes: map[string]corev1.Volume{
+			"the-name": {
+				Name: "the-name",
+				VolumeSource: corev1.VolumeSource{
+					ConfigMap: &corev1.ConfigMapVolumeSource{
+						LocalObjectReference: corev1.LocalObjectReference{
+							Name: "test-cm",
+						},
+					},
+				},
+			},
+		},
 		want: (&apis.FieldError{
 			Message: `mountPath "/var/log" is a reserved path`,
 			Paths:   []string{"mountPath"},
@@ -1072,8 +1162,38 @@ func TestContainerValidation(t *testing.T) {
 				ReadOnly:  true,
 			}},
 		},
-		volumes: sets.NewString("the-name"),
-		want:    apis.ErrInvalidValue("not/absolute", "volumeMounts[0].mountPath"),
+		volumes: map[string]corev1.Volume{
+			"the-name": {
+				Name: "the-name",
+				VolumeSource: corev1.VolumeSource{
+					ConfigMap: &corev1.ConfigMapVolumeSource{
+						LocalObjectReference: corev1.LocalObjectReference{
+							Name: "test-cm",
+						},
+					},
+				},
+			},
+		},
+		want: apis.ErrInvalidValue("not/absolute", "volumeMounts[0].mountPath"),
+	}, {
+		name: "Empty dir has rw access",
+		c: corev1.Container{
+			Image: "foo",
+			VolumeMounts: []corev1.VolumeMount{{
+				MountPath: "/mount/path",
+				Name:      "the-name",
+			}},
+		},
+		volumes: map[string]corev1.Volume{
+			"the-name": {
+				Name: "the-name",
+				VolumeSource: corev1.VolumeSource{
+					EmptyDir: &corev1.EmptyDirVolumeSource{
+						Medium: "Memory",
+					},
+				},
+			},
+		},
 	}, {
 		name: "has lifecycle",
 		c: corev1.Container{
@@ -1095,7 +1215,18 @@ func TestContainerValidation(t *testing.T) {
 				ReadOnly:  true,
 			}},
 		},
-		volumes: sets.NewString("the-name"),
+		volumes: map[string]corev1.Volume{
+			"the-name": {
+				Name: "the-name",
+				VolumeSource: corev1.VolumeSource{
+					ConfigMap: &corev1.ConfigMapVolumeSource{
+						LocalObjectReference: corev1.LocalObjectReference{
+							Name: "test-cm",
+						},
+					},
+				},
+			},
+		},
 	}, {
 		name: "valid with probes (no port)",
 		c: corev1.Container{
@@ -1265,6 +1396,29 @@ func TestContainerValidation(t *testing.T) {
 			},
 		},
 		want: apis.ErrDisallowedFields("securityContext.runAsGroup"),
+	}, {
+		name: "not allowed to add a security context capability",
+		c: corev1.Container{
+			Image: "foo",
+			SecurityContext: &corev1.SecurityContext{
+				Capabilities: &corev1.Capabilities{
+					Add: []corev1.Capability{"all"},
+				},
+			},
+		},
+		want: apis.ErrDisallowedFields("securityContext.capabilities.add"),
+	}, {
+		name: "allowed to add a security context capability when gate is enabled",
+		c: corev1.Container{
+			Image: "foo",
+			SecurityContext: &corev1.SecurityContext{
+				Capabilities: &corev1.Capabilities{
+					Add: []corev1.Capability{"all"},
+				},
+			},
+		},
+		cfgOpts: []configOption{withContainerSpecAddCapabilitiesEnabled()},
+		want:    nil,
 	}, {
 		name: "too large uid",
 		c: corev1.Container{
@@ -1462,9 +1616,10 @@ func TestContainerValidation(t *testing.T) {
 
 func TestVolumeValidation(t *testing.T) {
 	tests := []struct {
-		name string
-		v    corev1.Volume
-		want *apis.FieldError
+		name    string
+		v       corev1.Volume
+		want    *apis.FieldError
+		cfgOpts []configOption
 	}{{
 		name: "just name",
 		v: corev1.Volume{
@@ -1496,11 +1651,26 @@ func TestVolumeValidation(t *testing.T) {
 		v: corev1.Volume{
 			Name: "foo",
 			VolumeSource: corev1.VolumeSource{
-				EmptyDir: &corev1.EmptyDirVolumeSource{},
+				EmptyDir: &corev1.EmptyDirVolumeSource{
+					Medium:    "Memory",
+					SizeLimit: resource.NewQuantity(400, "G"),
+				},
 			},
 		},
-		want: apis.ErrMissingOneOf("secret", "configMap", "projected").Also(
-			apis.ErrDisallowedFields("emptyDir")),
+		cfgOpts: []configOption{withPodSpecVolumesEmptyDirEnabled()},
+	}, {
+		name: "invalid emptyDir volume",
+		v: corev1.Volume{
+			Name: "foo",
+			VolumeSource: corev1.VolumeSource{
+				EmptyDir: &corev1.EmptyDirVolumeSource{
+					Medium:    "Memory",
+					SizeLimit: resource.NewQuantity(-1, "G"),
+				},
+			},
+		},
+		want:    apis.ErrInvalidValue(-1, "emptyDir.sizeLimit"),
+		cfgOpts: []configOption{withPodSpecVolumesEmptyDirEnabled()},
 	}, {
 		name: "no volume source",
 		v: corev1.Volume{
@@ -1717,7 +1887,15 @@ func TestVolumeValidation(t *testing.T) {
 
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
-			got := validateVolume(test.v)
+			ctx := context.Background()
+			if test.cfgOpts != nil {
+				cfg := config.FromContextOrDefaults(ctx)
+				for _, opt := range test.cfgOpts {
+					cfg = opt(cfg)
+				}
+				ctx = config.ToContext(ctx, cfg)
+			}
+			got := validateVolume(ctx, test.v)
 			if diff := cmp.Diff(test.want.Error(), got.Error()); diff != "" {
 				t.Errorf("validateVolume (-want, +got): \n%s", diff)
 			}
@@ -1784,7 +1962,7 @@ func TestObjectReferenceValidation(t *testing.T) {
 			Kind:       "Bar",
 			Name:       "bad name",
 		},
-		want: apis.ErrInvalidValue("a DNS-1123 label must consist of lower case alphanumeric characters or '-', and must start and end with an alphanumeric character (e.g. 'my-name',  or '123-abc', regex used for validation is '[a-z0-9]([-a-z0-9]*[a-z0-9])?')", "name"),
+		want: apis.ErrInvalidValue("a lowercase RFC 1123 label must consist of lower case alphanumeric characters or '-', and must start and end with an alphanumeric character (e.g. 'my-name',  or '123-abc', regex used for validation is '[a-z0-9]([-a-z0-9]*[a-z0-9])?')", "name"),
 	}, {
 		name: "disallowed fields",
 		r: &corev1.ObjectReference{
